# from imio.smartweb.common.browser.forms import CustomAddForm
from imio.smartweb.common.ia.browser.categorization_button_add import (
    IACategorizeAddForm as BaseIACategorizeAddForm,
)
from plone.dexterity.browser.add import DefaultAddView


class IACategorizeAddForm(BaseIACategorizeAddForm):

    def update(self):
        return super(IACategorizeAddForm, self).update()


class IACategorizeAddView(DefaultAddView):
    form = IACategorizeAddForm



from collective.exportimport.import_content import ImportContent
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisiteSettings
from imio.smartweb.core.contents.pages.pages import IDefaultPages
from plone.dexterity.interfaces import IDexterityFTI
from six.moves.urllib.parse import unquote
from six.moves.urllib.parse import urlparse
from zExceptions import NotFound
from zope.component import getUtility
from zope.interface import alsoProvides
from plone import api

import logging

logger = logging.getLogger(__name__)
from Products.CMFCore.utils import getToolByName


class CustomImportContent(ImportContent):

    def cleanup_broken_brains(self, context, root_path, depth=3):
        catalog = getToolByName(context, "portal_catalog")
        brains = catalog(path={"query": root_path, "depth": depth})
        removed = 0
        for b in brains:
            try:
                b._unrestrictedGetObject()
            except (NotFound, KeyError, AttributeError):
                catalog.uncatalog_object(b.getPath())
                removed += 1
        return removed

    def create_container(self, item):
        """Create container for item.
        See remarks in get_parent_as_container for some corner cases.
        """
        folder = self.context
        parent_url = unquote(item["parent"]["@id"])
        parent_url_parsed = urlparse(parent_url)
        # Get the path part, split it, remove the always empty first element.
        parent_path = parent_url_parsed.path.split("/")[1:]
        if (
            len(parent_url_parsed.netloc.split(":")) > 1
            or parent_url_parsed.netloc == "nohost"
        ):
            # For example localhost:8080, or nohost when running tests.
            # First element will then be a Plone Site id.
            # Get rid of it.
            parent_path = parent_path[1:]

        # Handle folderish Documents provided by plone.volto
        fti = getUtility(IDexterityFTI, name="Document")
        # if "fr/la-boverie/album/section-gallery/logo-laboverie.jpg" in item["@id"]:
        #     import pdb;pdb.set_trace()
        parent_type = (
            "Document"
            if fti.klass.endswith("FolderishDocument")
            else item["parent"]["@type"]
        )
        # parent_type = (
        #    "Document" if fti.klass.endswith("FolderishDocument") else "Folder"
        # )
        # create original structure for imported content
        try:
            for element in parent_path:
                if element in folder:
                    folder = folder[element]
                    continue

                try:
                    new_folder = api.content.create(
                        container=folder,
                        type=parent_type,
                        id=element,
                        title=element,
                    )
                    folder = new_folder

                except NotFound:
                    # L'objet peut avoir été ajouté dans le BTree du folder
                    # mais l'exception vient souvent d'un event/reindex ensuite.
                    if element in folder:
                        root_path = "/".join(
                            folder.getPhysicalPath()
                        )  # le container courant
                        removed = self.cleanup_broken_brains(folder, root_path, depth=5)
                        logger.warning(
                            "Removed %s broken catalog entries under %s",
                            removed,
                            root_path,
                        )

                        folder = folder[element]
                        continue

                    # Si l'objet n'est même pas là, c'est un vrai échec
                    logger.error(
                        "NotFound during create and %s was NOT created in %s",
                        element,
                        folder.absolute_url(),
                    )
                    raise
        except:
            logger.error(
                f"ERREUR SUR :{element} -- {folder} -- {folder.absolute_url()}"
            )
        return folder

    def global_obj_hook(self, obj, item):
        """Inspect the content item before serialization data."""
        logger.info("imio.smartweb.core : global_obj_hook")
        # if (
        #     "vie-communale/projet-de-ville/liege-2030/projet-de-ville-liege-2030"
        #     in item["@id"]
        # ):
        #     # il faut regarder pourquoi l'image ne va faire que 1ko lors de l'importation
        #     import pdb;pdb.set_trace()
        if "cpskin.minisite.interfaces.IMinisiteRoot" in item.get(
            "_cpskin_interfaces", []
        ):
            alsoProvides(obj, IImioSmartwebMinisite)
            alsoProvides(obj, IImioSmartwebMinisiteSettings)
            setattr(obj, "logo", None)
            setattr(obj, "logo_display_mode", "title")
            logger.info(f"Set minisite to {obj.absolute_url()} ")

        if "imio.smartweb.core.contents.pages.pages.IDefaultPages" in item.get(
            "_smartweb_interfaces", []
        ):
            alsoProvides(obj, IDefaultPages)
            obj.aq_parent.set_default_item(new_default_item=obj)
            logger.info(
                f"Set INTERFACE 'cause I'm a default page : ({obj.absolute_url()}) "
            )
        # layout = item.get("layout")
        # if "fr/decouvrir/plein-air/art-public/la-fontaine-de-la-tradition-1" in item["@id"]:
        #     import pdb; pdb.set_trace()
        # if layout:
        #     try:
        #         obj.setLayout(layout)
        #         logger.info(f"  {layout} to {obj.absolute_url()} ")
        #         # print(f"setLayout {layout} TO {obj.absolute_url()}")
        #     except AttributeError:
        #         pass
        # if item.get("_cpskin_default_page", None):
        #     obj.setDefaultPage(item.get("_cpskin_default_page"))
        #     logger.info(f"Set default page to {obj.absolute_url()} ")
        obj.reindexObject()
        return obj

    # !!! by default, this method remove item layout
    def global_dict_hook(self, item):
        return item



# -*- coding: utf-8 -*-
from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.locales import SmartwebMessageFactory as _
from imio.smartweb.core.utils import get_json
from imio.smartweb.core.contents.rest.view import BaseRestView
from imio.smartweb.core.interfaces import IOgpViewUtils
from plone import api
from urllib.parse import parse_qs
from zope.interface import implementer


@implementer(IOgpViewUtils)
class DirectoryViewView(BaseRestView):
    """DirectoryView view"""

    @property
    def propose_url(self):
        return api.portal.get_registry_record("smartweb.propose_directory_url")

    @property
    def display_map(self):
        return self.context.display_map

    @property
    def contact(self):
        contact = self._formated_contact(self.item)
        return contact

    def _formated_contact(self, data):
        current_lang = api.portal.get_current_language()[:2]

        # Récupération des données de base
        name = data.get(f"title_{current_lang}") or data.get("title")
        subtitle = data.get(f"subtitle_{current_lang}") or data.get("subtitle")
        number = data.get("number")
        street = data.get("street")
        city = data.get("city")
        zipcode = data.get("zipcode")
        country = data.get("country").get("title") if data.get("country") else None
        full_address = self._format_address(street, number, zipcode, city, country)

        # Téléphone (premier numéro s’il existe)
        phone = None
        if data.get("phones"):
            phone = data["phones"][0].get("number")

        # Email (première adresse s’il existe)
        email = None
        if data.get("mails"):
            email = data["mails"][0].get("mail_address")

        # URL (premier lien s’il existe)
        url = None
        if data.get("urls"):
            url = data["urls"][0].get("url")

        # Description (française en priorité)
        description = data.get(f"description_{current_lang}") or data.get("description")

        # Catégorie
        category = None
        if data.get("taxonomy_contact_category"):
            category = data["taxonomy_contact_category"][0].get("title")

        # Type
        contact_type = data.get("type", {}).get("title")

        # Géolocalisation
        geo = data.get("geolocation") or {}
        latitude = geo.get("latitude")
        longitude = geo.get("longitude")

        # Picture
        image_info = data.get("image") or {}
        image_url = None

        # Choisir dynamiquement la scale 'portrait_affiche' si disponible
        if "scales" in image_info and "portrait_affiche" in image_info["scales"]:
            image_url = image_info["scales"]["portrait_affiche"]["download"]
        elif "download" in image_info:
            # Fallback sur l'image originale
            image_url = image_info["download"]

        # Construction du JSON simplifié
        prefix_address = _("Address")
        prefix_email = _("Email")
        prefix_description = _("Description")
        prefix_category = _("Contact category")
        prefix_type = _("Contact type")
        contact = {
            "name": name,
            "subtitle": subtitle,
            "address": f"{prefix_address}: {full_address}" if full_address else None,
            "phone": phone,
            "email": f"{prefix_email}: {email}" if email else None,
            "description": (
                f"{prefix_description}: {description}" if description else None
            ),
            "url": url,
            "category": f"{prefix_category}: {category}" if category else None,
            "geolocation": (latitude, longitude) if geo else None,
            "contact_type": f"{prefix_type}: {contact_type}" if contact_type else None,
            "image_url": image_url if image_url else None,
        }
        return contact



from imio.smartweb.common.ia.browser.views import BaseProcessCategorizeContentView
from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents import ISectionText
from imio.smartweb.core.utils import get_categories

import json


class ProcessCategorizeContentView(BaseProcessCategorizeContentView):

    def __call__(self):
        self._get_all_text()
        return super(ProcessCategorizeContentView, self).__call__()

    def _get_all_text(self):
        all_text = ""
        if IPages.providedBy(self.context):
            for _id, obj in self.context.objectItems():
                if ISectionText.providedBy(obj):
                    all_text += " " + getattr(obj.text, "output", "")
        else:
            # We don't have context so we're in ++add++
            # Need to get page title on request
            raw = self.request.get("BODY")
            data = json.loads(raw)
            all_text = data.get("formdata").get("form-widgets-IBasic-title", "")
        return all_text.strip()

    def _process_specific(self, all_text, results):
        """Must be impleted"""
        ia_categories = self._process_category(all_text, results)
        results["form-widgets-page_category-taxonomy_page_category"] = ia_categories
        return results

    def _process_category(self, all_text, results):
        # If agent doesn't register prevsouly a categorization so IA can bring categorization
        # ++add++ context is Plone and hasn't got taxonomy_page_category property
        if getattr(self.context, "taxonomy_page_category", None) is None:
            categories_taxo = get_categories()
            categories_voca = categories_taxo.makeVocabulary(self.current_lang).inv_data
            categories_voc = [
                {"title": v, "token": k} for k, v in categories_voca.items()
            ]
            data = self._ask_categorization_to_ia(all_text, categories_voc)
            if not data:
                return ""
            ia_categories = [
                {"title": r.get("title"), "token": r.get("token")}
                for r in data.get("result")
            ]
            return ia_categories



import logging
import os

log = logging.getLogger(__name__)

if os.environ.get("PROJECT_ID", "") == "liege_smartweb":
    from plone.namedfile.file import NamedBlobImage
    from plone.namedfile.file import MAX_INFO_BYTES
    from plone.namedfile.file import NamedBlobFile  # Import du parent
    from plone.namedfile.utils import getImageInfo

    def patched_setData(self, data):
        # 1. On appelle le parent direct (NamedBlobFile) pour stocker les données
        NamedBlobFile._setData(self, data)
        # 2. On récupère les premiers bytes pour l'analyse
        firstbytes = self.getFirstBytes()

        # 3. Analyse initiale
        res = getImageInfo(data)

        if (
            res == ("image/jpeg", -1, -1)
            or res == ("image/tiff", -1, -1)
            or res == ("image/svg+xml", -1, -1)
        ):
            # header was longer than firstbytes
            start = len(firstbytes)
            length = max(0, MAX_INFO_BYTES - start)
            firstbytes += self.getFirstBytes(start, length)
            res = getImageInfo(firstbytes)

        contentType, self._width, self._height = res
        if contentType:
            self.contentType = contentType

        log.info("Patch NamedBlobImage._setData exécuté pour %s", self.filename)

    # Application du patch
    NamedBlobImage._setData = patched_setData
else:
    log.info("Monkeypatch NamedBlobImage._setData NOT activated")



# -*- coding: utf-8 -*-
from datetime import date
from imio.smartweb.core.browser.sitemap import format_sitemap_items
from imio.smartweb.core.browser.sitemap import get_endpoint_data
from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.config import EVENTS_URL
from imio.smartweb.core.config import NEWS_URL
from imio.smartweb.core.contents import IDirectoryView
from imio.smartweb.core.contents import IEventsView
from imio.smartweb.core.contents import INewsView
from imio.smartweb.core.interfaces import INoIndexedUtils
from imio.smartweb.core.interfaces import IViewWithoutLeadImage
from imio.smartweb.core.utils import get_json
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from Products.Five import BrowserView
from urllib.parse import parse_qs
from urllib.parse import urlsplit
from zope.interface import implementer


@implementer(IViewWithoutLeadImage)
class BaseRestView(BrowserView):
    @property
    def batch_size(self):
        return self.context.nb_results

    @property
    def local_query_url(self):
        base_url = self.context.absolute_url()
        return f"{base_url}/@results"

    @property
    def local_filters_query_url(self):
        base_url = self.context.absolute_url()
        return f"{base_url}/@results-filters"

    @property
    def orientation(self):
        return self.context.orientation

    @property
    def current_language(self):
        return api.portal.get_current_language()[:2]

    @property
    def context_authenticated_user(self):
        return api.user.is_anonymous()

    @property
    def view_path(self):
        url = self.context.absolute_url()
        parsed = urlsplit(url)
        return url.replace(f"{parsed.scheme}://{parsed.netloc}", "")

    @property
    def direct_access(self):
        query_string = self.request.get("QUERY_STRING", "")
        params = parse_qs(query_string)
        uuid = params.get("u", [None])[0]
        if uuid and self.request.HTTP_REFERER == "":
            endpoint = "@search"
            if IEventsView.providedBy(self.context):
                endpoint = "@events"
                url = EVENTS_URL
            if IDirectoryView.providedBy(self.context):
                url = DIRECTORY_URL
            if INewsView.providedBy(self.context):
                url = NEWS_URL
            url = f"{url}/{endpoint}?UID={uuid}&fullobjects=0"
            self._item = get_json(url)
            return True
        return False

    @property
    def item(self):
        item = self._item.get("items", [])[0] if self._item else None
        return item

    def _format_address(
        self, street=None, number=None, zipcode=None, city=None, country=None
    ):
        street_part = ", ".join(filter(None, [street, number]))
        city_part = " ".join(filter(None, [str(zipcode) if zipcode else None, city]))
        address_parts = [street_part, city_part]
        address = " - ".join(filter(None, address_parts))
        if country:
            address += f" / {country}"
        return address


@implementer(INoIndexedUtils)
class SeoHiddenReactLinks(BrowserView):

    DEFAULT_BATCH_SIZE = 100
    _total = 0

    def __call__(self):
        b_start = int(self.request.form.get("b_start", 0))
        b_size = int(self.request.form.get("b_size", self.DEFAULT_BATCH_SIZE))

        if IEventsView.providedBy(self.context):
            today = date.today().isoformat()
            self.request.form["event_dates.range"] = "min"
            self.request.form["event_dates.query"] = today

        # Inject batching params
        self.request.form["b_start"] = b_start
        self.request.form["b_size"] = b_size

        data = get_endpoint_data(self.context, self.request)
        self.items = format_sitemap_items(
            data.get("items", []), self.context.absolute_url()
        )

        self._total = data.get("items_total", len(self.items))
        self.b_start = b_start
        self.b_size = b_size
        return self.index()

    @property
    def default_view(self):
        return self.context.absolute_url()

    @property
    def label(self):
        label = ""
        if IDirectoryView.providedBy(self.context):
            label = _("Direcotry : SEO links")
        elif IEventsView.providedBy(self.context):
            label = _("Agenda : SEO links")
        elif INewsView.providedBy(self.context):
            label = _("News : SEO links")
        return label

    @property
    def get_data(self):
        return getattr(self, "items", [])

    @property
    def total(self):
        """Total number of items in the current view."""
        return getattr(self, "_total", 0)



# -*- coding: utf-8 -*-

from imio.smartweb.common.browser.forms import CustomEditForm
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.contents import ISectionEvents
from imio.smartweb.core.contents import ISectionNews
from imio.smartweb.core.contents import ISectionTimestampedPublications
from imio.smartweb.core.contents import IDefaultPages
from plone.z3cform import layout
from z3c.form.interfaces import DISPLAY_MODE
from zope.schema.vocabulary import getVocabularyRegistry


class SmartwebCustomEditForm(CustomEditForm):
    def updateFields(self):
        super(SmartwebCustomEditForm, self).updateFields()
        if IDefaultPages.providedBy(self.context) or IImioSmartwebMinisite.providedBy(
            self.context
        ):
            # We need to make exclude_from_nav field read only as it is used
            # for default pages to exclude them from sitemap / navigation / ...
            # and for minisites to exclude them as well.
            for group in self.groups:
                if "IExcludeFromNavigation.exclude_from_nav" in group.fields:
                    group.fields["IExcludeFromNavigation.exclude_from_nav"].mode = (
                        DISPLAY_MODE
                    )


SmartwebCustomEditView = layout.wrap_form(SmartwebCustomEditForm)


class SectionBaseCustomEditForm(CustomEditForm):
    """Base form that can clean outdated values in specific_related_events."""

    _interface = None
    _field_name = None

    def _get_adapter(self):
        if self._interface is None:
            return None
        return self._interface(self.context, None)

    def _clean_specific_related_events(self, obj):
        """WEB-4338: Clean missing (outdated) events/news/publications in field : self._field_name."""
        values = list(getattr(obj, self._field_name, None) or [])
        if not values:
            return

        field = self.fields[self._field_name].field
        vt = field.value_type
        vocab = getVocabularyRegistry().get(self.context, vt.vocabularyName)
        kept = []
        for uid in values:
            try:
                vocab.getTerm(uid)
                kept.append(uid)
            except LookupError:
                getattr(obj, self._field_name, None).remove(uid)

    def updateFields(self):
        super(SectionBaseCustomEditForm, self).updateFields()
        obj = self._get_adapter()
        if obj is None:
            return
        self._clean_specific_related_events(obj)


class SectionEventsCustomEditForm(SectionBaseCustomEditForm):
    _interface = ISectionEvents
    _field_name = "specific_related_events"


SectionEventsCustomEditView = layout.wrap_form(SectionEventsCustomEditForm)


class SectionNewsCustomEditForm(SectionBaseCustomEditForm):
    _interface = ISectionNews
    _field_name = "specific_related_newsitems"


SectionNewsCustomEditView = layout.wrap_form(SectionNewsCustomEditForm)


class SectionPublicationsCustomEditForm(SectionBaseCustomEditForm):
    _interface = ISectionTimestampedPublications
    _field_name = "related_timestamped_publications"


SectionPublicationsCustomEditView = layout.wrap_form(SectionPublicationsCustomEditForm)



# -*- coding: utf-8 -*-
from imio.smartweb.common.ia.browser.categorization_button_edit import (
    IACategorizeEditForm,
)
from imio.smartweb.core.browser.forms import SmartwebCustomEditForm
from plone.z3cform import layout
from z3c.form.interfaces import HIDDEN_MODE


FIELD_NAME = "categorization_ia_link"  # Internal id for dummy field


class PageEditForm(SmartwebCustomEditForm, IACategorizeEditForm):
    """Vue edit custom, avec bouton 'Catégoriser' injecté en haut de 'categorization'."""

    def update(self):
        # Hide hide_title in 'layout' group
        for group in getattr(self, "groups", []):
            if (
                getattr(group, "__name__", "") == "layout"
                and "hide_title" in group.widgets
            ):
                group.widgets["hide_title"].mode = HIDDEN_MODE
                group.widgets["hide_title"].value = ["selected"]
        super(PageEditForm, self).update()


PageEditView = layout.wrap_form(PageEditForm)
